'''
pyutillib/string_utils.py

Copyright (C) 2013 Edwin van Opstal

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see `<http://www.gnu.org/licenses/>`.
'''

from __future__ import division
from __future__ import absolute_import

import ast
import random
import string


def random_string(length=8, charset=None):
    '''
    Generates a string with random characters. If no charset is specified, only
    letters and digits are used.

    Args:
        length (int) length of the returned string
        charset (string) list of characters to choose from
    Returns:
        (str) with random characters from charset
    Raises:
        -
    '''
    if length < 1:
        raise ValueError('Length must be > 0')
    if not charset:
        charset = string.letters + string.digits
    return ''.join(random.choice(charset) for unused in xrange(length))


def safe_eval(str_in):
    '''
    Extracts a python object from a string.

    Args:
        str_in (string) that contains python variable
    Returns:
        (object) of standard python type or None of no valid object was found.
    Raises:
        -
    '''
    try:
        return ast.literal_eval(str_in)
    except:
        return None


def str2dict(str_in):
    '''
    Extracts a dict from a string.

    Args:
        str_in (string) that contains python dict
    Returns:
        (dict) or None if no valid dict was found
    Raises:
        -
    '''
    dict_out = safe_eval(str_in)
    if not isinstance(dict_out, dict):
        dict_out = None
    return dict_out


def str2tuple(str_in):
    '''
    Extracts a tuple from a string.

    Args:
        str_in (string) that contains python tuple
    Returns:
        (dict) or None if no valid tuple was found
    Raises:
        -
    '''
    tuple_out = safe_eval(str_in)
    if not isinstance(tuple_out, tuple):
        tuple_out = None
    return tuple_out


#used to be get_dict_keys
def str2dict_keys(str_in):
    '''
    Extracts the keys from a string that represents a dict and returns them
    sorted by key.

    Args:
        str_in (string) that contains python dict
    Returns:
        (list) with keys or None if no valid dict was found
    Raises:
        -
    '''
    tmp_dict = str2dict(str_in)
    if tmp_dict is None:
        return None
    return sorted([k for k in tmp_dict])


#used to be get_dict_values
def str2dict_values(str_in):
    '''
    Extracts the values from a string that represents a dict and returns them
    sorted by key.

    Args:
        str_in (string) that contains python dict
    Returns:
        (list) with values or None if no valid dict was found
    Raises:
        -
    '''
    tmp_dict = str2dict(str_in)
    if tmp_dict is None:
        return None
    return [tmp_dict[key] for key in sorted(k for k in tmp_dict)]



'''
pyutillib/tests

Copyright (C) 2013 Edwin van Opstal

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see `<http://www.gnu.org/licenses/>`.
'''

from __future__ import division
from __future__ import absolute_import

import unittest
import datetime as dt

import pyutillib.date_utils as du
import pyutillib.math_utils as mu
import pyutillib.string_utils as su


class TestDateUtils(unittest.TestCase):
    '''
    '''

    def setUp(self):
        dd = dt.date
        self.validdata = (
            {'date': dd(2000,1,31), 'str': '20000131', 'fmt': 'yyyymmdd'},
            {'date': dd(2000,1,31), 'str': '000131', 'fmt': 'yymmdd'},
            {'date': dd(2000,1,31), 'str': '31-1-00', 'fmt': 'd-m-yy'},
            {'date': dd(2000,1,31), 'str': '31-1-2000', 'fmt': 'd-m-yyyy'},
            {'date': dd(2000,1,31), 'str': '31-01-00', 'fmt': 'dd-mm-yy'},
            {'date': dd(2000,1,31), 'str': '31-01-2000', 'fmt': 'dd-mm-yyyy'},
            {'date': dd(2000,1,31), 'str': '1/31/00', 'fmt': 'm/d/yy'},
            {'date': dd(2000,1,31), 'str': '1/31/2000', 'fmt': 'm/d/yyyy'},
            {'date': dd(2000,1,31), 'str': '01/31/00', 'fmt': 'mm/dd/yy'},
            {'date': dd(2000,1,31), 'str': '01/31/2000', 'fmt': 'mm/dd/yyyy'},
            )
        self.weekdays = (dd(2013,2,22), dd(2013,2,25), dd(2013,2,26), 
                dd(2013,2,27), dd(2013,2,28), dd(2013,3,1), dd(2013,3,4))
        self.weekend = (dd(2013,2,23), dd(2013,2,24), dd(2013,3,2), 
                dd(2013,3,3))

    def test_datestr2date(self):
        for data in self.validdata:
            self.assertEqual(du.datestr2date(data['str']), data['date'])

        invaliddateformats = ('1', '11', '111', '1111', '11111', '1111111',
            '111111111', '1111111111', '1111011', '1101111', 
            '031-1-00', '31-001-00', '31-1-000', '31-1-20001',
            'a1-1-00',
            )
        for date in invaliddateformats:
            self.assertRaises(ValueError, du.datestr2date, date)

        invaliddates = ('31/1/00', '01-31-00', '20010229')
        for date in invaliddates:
            self.assertRaises(ValueError, du.datestr2date, date)


    def test_date2datestr(self):
        #default fmt:
        self.assertEqual(du.date2datestr(self.validdata[0]['date']), 
                    self.validdata[0]['str'])
        #with format specifier
        for data in self.validdata:
            self.assertEqual(du.date2datestr(data['date'], data['fmt']), 
                    data['str'])
        invalidformats = ('dmy', 'd/m/y', 'dd/mm/yy', 'mm-dd-yy', 'yy-mm-dd',)
        date = dt.date(2000, 1, 31)
        for fmt in invalidformats:
            self.assertRaises(ValueError, du.date2datestr, date, fmt)


    def test_is_weekday(self):
        for date in self.weekdays:
            self.assertTrue(du.is_weekday(date))
        for date in self.weekend:
            self.assertFalse(du.is_weekday(date))


    def test_is_weekend(self):
        for date in self.weekdays:
            self.assertFalse(du.is_weekend(date))
        for date in self.weekend:
            self.assertTrue(du.is_weekend(date))


    def test_previous_weekday(self):
        for day1, day2 in ((8,7), (9,7), (10,7), (11,10), (12,11), (13,12), 
                           (14,13), (15,14), (16,14), (17,14)):
            self.assertEqual(du.previous_weekday(dt.date(2000,1,day1)), 
                    dt.date(2000,1,day2))


    def test_next_weekday(self):
        for day1, day2 in ((6, 7), (7, 10), (8,10), (9,10), (10,11), (11,12), 
                           (12,13), (13,14), (14,17), (15,17), (16,17)):
            self.assertEqual(du.next_weekday(dt.date(2000,1,day1)), 
                    dt.date(2000,1,day2))


    def test_last_year(self):
        self.assertEqual(du.last_year(dt.date(2000,3,29)), dt.date(1999,3,29))
        self.assertEqual(du.last_year(dt.date(2000,2,27)), dt.date(1999,2,27))
        self.assertEqual(du.last_year(dt.date(2000,2,29)), dt.date(1999,2,28))


class TestMathUtils(unittest.TestCase):

    def setUp(self):
        pass


    def test_div(self):
        self.assertEqual(mu.div(0,0), 0)
        self.assertEqual(mu.div(0,1), 0)
        self.assertEqual(mu.div(1,0), float('inf'))
        self.assertEqual(mu.div(2,1), 2.)
        self.assertEqual(mu.div(2,1.25), 1.6)
        self.assertEqual(mu.div(2,0.5), 4.)
        self.assertEqual(mu.div(0.5,0.25), 2.)
        self.assertEqual(mu.div(0.2, 0.1), 2.)


    def test_eval_conditions(self):
        self.assertTrue(mu.eval_conditions(None))
        self.assertRaises(TypeError, mu.eval_conditions, 'a')
        self.assertRaises(TypeError, mu.eval_conditions, (1,2,3,4))

        # simple conditions
        for conditions in ( (2, 'gt', 1), 
                            ('a', 'eq', 'a'),
                            ('a', 'lt', 'b'),
                            (2., 'gt', 1),
                            (1, 'eq', 1.),
                            (True, 'eq', True),
                            (True, 'or', False),
                            ):
            self.assertTrue(mu.eval_conditions(conditions), conditions)
        for conditions in ( (2, 'lt', 1), 
                            ('a', 'eq', 'b'),
                            (3, 'gt', 4.2),
                            (True, 'eq', False),
                            (True, 'and', False),
                            ):
            self.assertFalse(mu.eval_conditions(conditions), conditions)

        for conditions in ( 
                ('a', 'eq', 1),
                (True, 'eq', 3),
                (3, 'and', True),
                ):
            self.assertRaises(TypeError, mu.eval_conditions, conditions)

        for conditions in (
                (1, 'abc', 2),
                (1, 3, 2),
                ):
            self.assertRaises(ValueError, mu.eval_conditions, conditions)

        # simple conditions with data
        for conditions, data in ( (('x', 'lt', 2), {'x': 1}),
                                  (('y', 'eq', 1.3), {'y': 1.3}),
                                  (('y', 'and', True), {'y': True}),
                                 ):
            self.assertTrue(mu.eval_conditions(conditions, data), conditions)
        for conditions, data in ( (('x', 'lt', 1), {'x': 2}),
                                  (('x', 'eq', 1), {'x': 1.1}),
                                  ((True, 'and', 'abc'), {'abc': False}),
                                 ):
            self.assertFalse(mu.eval_conditions(conditions, data), conditions)
        for conditions, data in (
                (('x', 'gt', 1), {'y': 2}),
                ):
            self.assertRaises(TypeError, mu.eval_conditions, conditions, data)

        for conditions, data in (
                (('x', 'abc', 1), {'abc': 'and'}),
                ):
            self.assertRaises(ValueError, mu.eval_conditions, conditions, data)

        # complex conditions
        for conditions, data in (
                ((True, 'and', ('x', 'eq', 2)), {'x': 2}),
                (('x', 'eq', (1, 'ne', 'y')), {'x': True, 'y': 2}),
                                 ):
            self.assertTrue(mu.eval_conditions(conditions, data), conditions)
        for conditions, data in (
                ((('x', 'lt', 1), 'and', (2, 'lt', 'y')), {'x': 2, 'y': 1}),
                                 ):
            self.assertFalse(mu.eval_conditions(conditions, data), conditions)
        for conditions, data in (
                ((2, ('lt' , 'or', 'gt'), 1), {}),
                (('x', 'abc', 1), {'abc': 'and'}),
                                 ):
            self.assertRaises(ValueError, mu.eval_conditions, conditions, data)


class TestStringUtils(unittest.TestCase):

    def setUp(self):
        pass


    def test_random_string(self):
        # test default length
        self.assertEqual(len(su.random_string()), 8)
        # test length setting
        for length in xrange(5,10):
            self.assertEqual(len(su.random_string(length)), length)
        # test invalid lengths
        for length in (-2, -1, 0):
            self.assertRaises(ValueError, su.random_string, length)
        # test uniqueness - could theoretically fail, but very unlikely
        testlist = (su.random_string() for unused in xrange(10))
        self.assertEqual(len(set(testlist)), 10)
        # test default charset
        for c in su.random_string(10000):
            self.assertNotIn(c, '~!@#$%^&*()_+`-={}|[]\\:";\'<>?,./')
        # test charset - could theoretically fail, but very unlikely
        charset = 'ABC'
        abc = su.random_string(100, charset)
        self.assertEqual(set(abc), {'A', 'B', 'C'})


    def test_safe_eval(self):
        self.assertEqual(su.safe_eval('15'), 15)
        self.assertEqual(su.safe_eval('8.45'), 8.45)
        self.assertEqual(su.safe_eval('(1,2)'), (1,2))
        self.assertEqual(su.safe_eval('[1,2,3]'), [1,2,3])
        self.assertEqual(su.safe_eval("{'a':1, 4:'asdf'}"), {'a':1, 4:'asdf'})
        for s in ('raise SystemExit', 'import sys\n', '', '1y3', '[3,1', ''):
            self.assertIsNone(su.safe_eval(s))


    def test_str2dict(self):
        self.assertEqual(su.str2dict("{'a':1, 4:'asdf'}"), {'a':1, 4:'asdf'})
        self.assertIsNone(su.str2dict("{'a':1 4:'asdf'}"))
        self.assertIsNone(su.str2dict('raise SystemExit'))
        self.assertIsNone(su.str2dict('(1,2,3)'))


    def test_str2tuple(self):
        self.assertEqual(su.str2tuple("('a', 4, [1,2,3])"), ('a', 4, [1,2,3]))
        self.assertIsNone(su.str2tuple("['a', 4, {1.4,}]"))
        self.assertIsNone(su.str2tuple('raise SystemExit'))
        self.assertIsNone(su.str2tuple('(1)'))


    def test_get_dict_keys_values(self):
        dict_string = '{"a":1, 2:"3", -1: 0}'
        self.assertEqual(su.str2dict_values(dict_string), [0, '3', 1])
        self.assertIsNone(su.str2dict_values('asdf'))
        self.assertEqual(su.str2dict_keys(dict_string), [-1, 2, 'a'])
        self.assertIsNone(su.str2dict_keys('asdf'))


if __name__ == '__main__':
    unittest.main()



'''
pyutillib/math_utils.py

Copyright (C) 2013 Edwin van Opstal

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see `<http://www.gnu.org/licenses/>`.
'''

from __future__ import division
from __future__ import absolute_import

import operator
from pyutillib.string_utils import str2tuple


def div(numerator, denominator):
    '''
    Returns numerator / denominator, but instead of a ZeroDivisionError:
        0 / 0 = 0.
        x / 0 = float('inf')
    This is not mathematically correct, but often practically OK.

    Args:
        numerator (float or int)
        denominator (float or int)
    Returns:
        (float)
    Raises:
        -
    '''
    try:
        return numerator/denominator
    except ZeroDivisionError:
        if numerator == 0:
            return 0.
        elif denominator == 0:
            return float('inf')
#            return None
        else:
            return numerator/denominator


def eval_conditions(conditions=None, data={}):
    '''
    Evaluates conditions and returns Boolean value.

    Args:
        conditions (tuple) for the format of the tuple, see below
        data (dict) the keys of which can be used in conditions
    Returns:
        (boolea)
    Raises:
        ValueError if an invalid operator value is specified
        TypeError if: 
                conditions are not a 3-item tuple
                the arguments of the condition don't have the same type
                    e.g. ('abc', 'eq', 3)
                if a boolean operator does not get boolean arguments
                    e.g. (True, 'and', 15)

    The format of the condition tuple is:
        (arg1, op, arg2)
    where:
        arg1, arg2 can be numerical values, strings or condition tuples
        op is a valid operator from the operator module
    If arg is a string, and the string is a key in <data> it is treated as
    a variable with value data[arg].

    Notes:
        * If no conditions are specified True is returned.
        * empty or 0 values do *not* evaluate to booleans
    '''
#CONSIDER: implementing addition/subtraction/multiplication/division
    if not conditions:
        return True
    if isinstance(conditions, str) or isinstance(conditions, unicode):
        conditions = str2tuple(conditions)
    if not isinstance(conditions, tuple) or not len(conditions) == 3:
        raise TypeError('conditions must be a tuple with 3 items.')
    arg1 = conditions[0]
    op = conditions[1]
    arg2 = conditions[2]
    if arg1 in data:
        arg1 = data[arg1]
    elif isinstance(arg1, tuple):
        arg1 = eval_conditions(arg1, data)
    if arg2 in data:
        arg2 = data[arg2]
    elif isinstance(arg2, tuple):
        arg2 = eval_conditions(arg2, data)
    if op in ('lt', 'le', 'eq', 'ne', 'ge', 'gt'):
        if not (type(arg1) in (float, int) and type(arg2) in (float,int)) and \
                type(arg1) != type(arg2):
            raise TypeError('both arguments must have the same type {}, {}'.\
                    format(arg1, arg2))
    elif op in ('and', 'or'):
        if not isinstance(arg1, bool) or not isinstance(arg2, bool):
            raise TypeError('boolean operator {} needs boolean arguments {},'\
                    ' {}'.format(op, arg1, arg2))
        op += '_'
    else:
        raise ValueError('operator {} not supported', op)
    return getattr(operator, op)(arg1, arg2)



'''
pyutillib/date_utils.py

Copyright (C) 2013 Edwin van Opstal

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see `<http://www.gnu.org/licenses/>`.
'''

from __future__ import division
from __future__ import absolute_import

import datetime


VALID_DATE_FORMATS_TEXT = '''The following date formats are valid:
    yymmdd    yyyymmdd
    d-m-yy    d-m-yyyy
    m/d/yy    m/d/yyyy
Where in the latter 4 formats m and d may be 1 or 2 digits which may include
a leading zero.
In case of a 2-digit year, it is assumed to be after 2000'''


def datestr2date(date_str):
    '''
    Turns a string into a datetime.date object. This will only work if the 
    format can be "guessed", so the string must have one of the formats from
    VALID_DATE_FORMATS_TEXT.

    Args:
        date_str (str) a string that represents a date
    Returns:
        datetime.date object
    Raises:
        ValueError if the input string does not have a valid format.
    '''
    if any(c not in '0123456789-/' for c in date_str):
        raise ValueError('Illegal character in date string')
    if '/' in date_str:
        try:
            m, d, y = date_str.split('/')
        except:
            raise ValueError('Date {} must have no or exactly 2 slashes. {}'.
                    format(date_str, VALID_DATE_FORMATS_TEXT))
    elif '-' in date_str:
        try:
            d, m, y = date_str.split('-')
        except:
            raise ValueError('Date {} must have no or exactly 2 dashes. {}'.
                    format(date_str, VALID_DATE_FORMATS_TEXT))
    elif len(date_str) == 8 or len(date_str) == 6:
        d = date_str[-2:]
        m = date_str[-4:-2]
        y = date_str[:-4]
    else:
        raise ValueError('Date format not recognised. {}'.format(
                VALID_DATE_FORMATS_TEXT))
    if len(y) == 2:
        year = 2000 + int(y)
    elif len(y) == 4:
        year = int(y)
    else:
        raise ValueError('year must be 2 or 4 digits')
    for s in (m, d):
        if 1 <= len(s) <= 2:
            month, day = int(m), int(d)
        else:
            raise ValueError('m and d must be 1 or 2 digits')
    try:
        return datetime.date(year, month, day)
    except ValueError:
        raise ValueError('Invalid date {}. {}'.format(date_str, 
                VALID_DATE_FORMATS_TEXT))


def date2datestr(date, fmt='yyyymmdd'):
    '''
    Turns a datetime.date object into a string. The string must have one of the
    formats from VALID_DATE_FORMATS_TEXT to make it compatible with 
    datestr2date.

    Args:
        date (datetime.date) the date to be translated
        fmt (str) a format string.
    Returns:
        (str) that represents a date.
    Raises:
        ValueError if the format is not valid.
    '''
    if '-' in fmt:
        if not fmt.index('d') < fmt.index('m') < fmt.index('y'):
            raise ValueError('Invalid format string. {}'.format(
                    VALID_DATE_FORMATS_TEXT))
        d, m, y = fmt.split('-')
    elif '/' in fmt:
        if not fmt.index('m') < fmt.index('d') < fmt.index('y'):
            raise ValueError('Invalid format string. {}'.format(
                    VALID_DATE_FORMATS_TEXT))
        m, d, y = fmt.split('/')
    elif any(c not in 'dmy' for c in fmt):
        raise ValueError('Invalid character in format string. {}'.format(
                VALID_DATE_FORMATS_TEXT))
    else:
        if not fmt.index('y') < fmt.index('m') < fmt.index('d'):
            raise ValueError('Invalid format string. {}'.format(
                    VALID_DATE_FORMATS_TEXT))
        y, m, d = fmt[:-4], fmt[-4:-2], fmt[-2:]
    for string, char in ((d, 'd'), (m, 'm'), (y, 'y')):
        if any(c != char for c in string):
            raise ValueError('Invalid date format: {} is not {}'.\
                    format(char, string))
    if len(y) == 4:
        fmt = fmt.replace('yyyy', '%Y', 1)
    elif len(y) == 2:
        fmt = fmt.replace('yy', '%y', 1)
    else:
        raise ValueError('Invalid format string, year must have 2 or 4 digits')
    if len(m) == 2:
        fmt = fmt.replace('mm', '%m', 1)
    elif len(m) == 1:
        fmt = fmt.replace('m', 'X%m', 1)
    else:
        raise ValueError('Invalid format string, month must have 1 or 2 digits')
    if len(d) == 2:
        fmt = fmt.replace('dd', '%d', 1)
    elif len(d) == 1:
        fmt = fmt.replace('d', 'X%d', 1)
    else:
        raise ValueError('Invalid format string, day must have 1 or 2 digits')
    return date.strftime(fmt).replace('X0','X').replace('X','')


def is_weekday(date):
    '''
    Returns a boolean that indicates if date is a weekday.

    Args:
        date (datetime or datetime.date)
    Returns:
        (boolean)
    Raises:
        -
    '''
    return True if date.weekday() < 5 else False


def is_weekend(date):
    '''
    Returns a boolean that indicates if date is in a weekend.

    Args:
        date (datetime or datetime.date)
    Returns:
        (boolean)
    Raises:
        -
    '''
    return not is_weekday(date)


def previous_weekday(date):
    '''
    Returns the last weekday before date

    Args:
        date (datetime or datetime.date)
    Returns:
        (datetime or datetime.date)
    Raises:
        -
    '''
    weekday = date.weekday()
    if weekday == 0:
        n_days = 3
    elif weekday == 6:
        n_days = 2
    else:
        n_days = 1
    return date - datetime.timedelta(days=n_days)


def next_weekday(date):
    '''
    Return the first weekday after date

    Args:
        date (datetime or datetime.date)
    Returns:
        (datetime or datetime.date)
    Raises:
        -
    '''
    n_days = 7 - date.weekday()
    if n_days > 3:
        n_days = 1
    return date + datetime.timedelta(days=n_days)


def last_year(date_):
    '''
    Returns the same date 1 year ago.

    Args:
        date (datetime or datetime.date)
    Returns:
        (datetime or datetime.date)
    Raises:
        -
    '''
    day = 28 if date_.day == 29 and date_.month == 2 else date_.day
    return datetime.date(date_.year-1, date_.month, day)



from distutils.core import setup
from setuptools import find_packages

setup(
    name='pyutillib',
    version='0.1.1',
    author='Edwin van Opstal',
    author_email='evo.se-technology.com',
    url='http://github.com/EdwinvO/pyutillib',
    license='LICENSE.txt',
    description='Small collection of python functions',
    long_description=open('README.rst').read(),
    install_requires=[
    ],
    packages=find_packages(),
)
